//	UpDnCounter_Testbed.v

/*	Testbed for Modulo-4 up/down counter.
 *
 *	Switch[9]: 		Up = Run; Down = Step.
 *	Key[3]:			Reset
 *	Key[2]:			Human Clock
 *	Key[1]:			Count Up
 *	Key[0]:			Count Down
 *	RedLED[9]:		On = Run; Off = Step
 *	GreenLED[7]:		Z output from counter
 *	GreenLED[6]:		Enable input to counter
 *	GreenLED[5:4]:	Counter state flip-flops
 *	GreenLED[3:0]:	Key_3..0
 *	Hex_0:
 *		Left two bits show state of Human Clock FSM:
 *			00 Waiting for Key_2 to be pressed
 *			01 human clock goes to 1
 *			10 human clock goes back to 0
 *			11 Waiting for Key_2 to be released
 *		Right two bits show the Counter FSM flip-flops
 *		(Same as GreenLED[5:4])
 *
 *	C. Vickery
 *	October, 2008
 */
 
module UpDnCounter_Testbed(
	input wire  [9:0] Switch,
	input wire  [3:0] Key,
	input wire        FPGA_Clock,
	output wire [9:0] RedLED,
	output wire [7:0] GreenLED,
	output wire [6:0] SevenSeg_0
	);

	//	Human clock states
	localparam [1:0]	awaiting_1	= 2'b00,
						turned_on	= 2'b01,
						turned_off  = 2'b10,
						awaiting_0	= 2'b11;

	wire [1:0] 	counter_state;
	wire		run, divided_clock, human_clock, enable, z;

	reg [1:0]	human_state;
	
	// 	Human clock: generate one pulse, the width of the FPGA clock period,
	//	each time Key 2 is pressed.
	always @ (posedge FPGA_Clock)
	case (human_state)
		awaiting_1: human_state = ~Key[2] ? turned_on : awaiting_1;
		turned_on: human_state = turned_off;
		turned_off: human_state = awaiting_0;
		awaiting_0: human_state = Key[2] ? awaiting_1 : awaiting_0;
	endcase

	//	 Display outputs
	assign RedLED = Switch;
	assign GreenLED[3:0] = ~Key;
	assign GreenLED[7] = z;
	assign GreenLED[6] = enable;
	assign GreenLED[5:4] = counter_state;
	
	//	Generate human clock pulses
	assign human_clock = human_state == turned_on;

	//	The Run/Step switch
	assign run = Switch[9];
	
	//	The internal enable signal.
	assign enable = run ? divided_clock : human_clock;

	//	Instantiate modules
	hex2sevenseg Result_Display(.Hex( {human_state, counter_state} ), .Segments (SevenSeg_0) );
    DivideByN    divide_by_n ( .In (FPGA_Clock), .Out (divided_clock));
	UpDnCounter  up_dn_counter (.Enable( enable ), .Clock( FPGA_Clock ), .Up( ~Key[1] ), .Dn( ~Key[0] ), .Reset( ~Key[3] ),
	                            .Z(z), .S1(counter_state[1]), .S0(counter_state[0]) );

endmodule
